%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%                       THE SURROGATE ENHANCED 
%                EVOLUTIONARY ANNEALING-SIMPLEX ALGORITHM
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% Developed by: Tsoukalas Ioannis and Kossieris Panagiotis
% 
% Based on the publication of:
%
% Tsoukalas, I., P. Kossieris, A. Efstratiadis, and C. Makropoulos,
% Surrogate-enhanced evolutionary annealing simplex algorithm for effective
% and efficient optimization of water resources problems on a budget,
% Environmental Modelling and Software, 77, 122142, doi:10.1016/j.envsoft.2015.12.008, 2016.  
%
% More: http://www.itia.ntua.gr/el/docinfo/1587/
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% SYNTAX:
%
% [BestValue,BestPar,MoreResults] = SEEAS(n,m,xmin,xmax,fn,maxeval,IntOptions,beta,maxclimbs,pmut,ftol)
%
%                            INPUT ARGUMENTS
%
%  n : problem dimension
%  m : population count ( m > n+1 )
%  xmin : lower parameter interior bounds
%  xmax : upper parameter interior bounds
%  xlow : lower parameter exterior bounds
%  xup : upper parameter exterior bounds
%  fn : objective function
%  maxeval : maximum number of function evaluations
%  ftol : the fractional convergence tolerance to be achieved in the function value for an early return
%  pmut : probability of accepting an offspring generated via mutation
%  beta : annealing schedule parameter
%  maxclimbs : maximum number of uphill steps
%  IntOptions (MATLAB structure) with the following elements:
%  "IntOptions" contains the parameters of the internal optimization algorithm
%  and the number of trials conducted by the surrogate assisted simplex.
%      - IntOptions.trials : number of trials conducted by the surrogate assisted simplex
%      - IntOptions.gapop : population size of the internal GA
%      - IntOptions.gagen : max generation number of the internal GA
%
%                             OUTPUT ARGUMENTS
%
%   BestValue : the minimal value of the objective function
%   BestPar : the optimal values of control variables
%
%   MATLAB structure (MoreResults) with the following elements:
%     - MoreResults.NumIter : number of iterations taken by algorithm
%     - MoreResults.Ftolpop : fractional convergence tolerance of population generated at the last iteration
%     - MoreResults.fmin: the minimum value of every iteration
%     - MoreResults.fperiter : cumulative function evaluation counter per iteration
%     - MoreResults.S : archive of parameters (decision variables) employed by the algorithm
%     - MoreResults.Y : archive of objective function values for parameters stored in S
%     - MoreResults.Temp : The temperature of annealing process
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% USAGE:
%
% Ackley function example

% with 6 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=500;
% [BestValue,BestPar,MoreResults] = SEEAS(10,50,xmin,xmax,@ackley,maxeval);

% with 7 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=500;
% IntOptions.trials=500;
% IntOptions.gapop=2*10;
% IntOptions.gagen=100;
% [BestValue,BestPar,MoreResults] = SEEAS(10,50,xmin,xmax,@ackley,maxeval,IntOptions)

% with 8 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=500;
% IntOptions.trials=500;
% IntOptions.gapop=2*10;
% IntOptions.gagen=100;
% [BestValue,BestPar,MoreResults] = SEEAS(10,50,xmin,xmax,@ackley,maxeval,IntOptions,3)

% with 9 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=500;
% IntOptions.trials=500;
% IntOptions.gapop=2*10;
% IntOptions.gagen=100;
% [BestValue,BestPar,MoreResults] = SEEAS(10,50,xmin,xmax,@ackley,maxeval,IntOptions,3,4)

% with 10 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=500;
% IntOptions.trials=500;
% IntOptions.gapop=2*10;
% IntOptions.gagen=100;
% [BestValue,BestPar,MoreResults] = SEEAS(10,50,xmin,xmax,@ackley,maxeval,IntOptions,3,4,0.1)

% with 11 input arguments
% xmin=ones(1,10)*-5.12; xmax=ones(1,10)*5.12;maxeval=500;
% IntOptions.trials=500;
% IntOptions.gapop=2*10;
% IntOptions.gagen=100;
% [BestValue,BestPar,MoreResults] = SEEAS(10,50,xmin,xmax,@ackley,maxeval,IntOptions,3,4,0.1,10^-6);
% 
% When a parameter is ommited the following default parameters are used:
% beta=3; maxclimbs=3; pmut=0.1; ftol=10^-6;
% IntOptions.trials=500;
% IntOptions.gapop=2*n;
% IntOptions.gagen=100;